#ifndef __MATRICE_H
#define __MATRICE_H

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include "Error.h"
#include "ReelInfini.h"

typedef struct Matrice * Matrice;


       /*
       Remarque: les indices commencent  0
       */

       /*Liste des erreurs possibles
        erreur601: Taille impossible
        erreur602: Erreur de la mmoire
        erreur603: Matrice non trouve
        erreur604: Les tailles des matrice ne sont pas compatibles
        erreur605: L'indice est hors borne
        erreur606: La matrice doit etre un vecteur ligne
        erreur607: L'erreur du calcul des ReelInfini
        erreur608: La matrice n'est pas factorisable.
        erreur609: La matrice n'est pas carre.
        erreur610: La matrice doit tre un  vecteur colonne
        */


//===================================================================================//
//Constructeur
//S'il y a un problme des donnes : retourne NULL et erreur601
//S'il y a un problme de la mmoire : retourne NULL et erreur602
//Retourne une Matrice si OK.
Matrice creerUneMatrice(int  hauteur, int longueur);

//Destructeur
//Retourne erreur 603 si m est introuvable
void detruireUneMatrice(Matrice m);

/*===============================Fonctions permettant d'obtenir les informations======*/

//==========
//Retourne -1 et erreur 603 si m est introuvable
int obtenirLaHauteur(Matrice m);

//==========
//Retourne -1 et erreur 603 si m est introuvable
int obtenirLaLongueur(Matrice m);

//==========
//Retourne erreur 603 si m est introuvable
ReelInfini obtenirLeDeterminant(Matrice m);

//==========
//Retourne erreur 603 si m est introuvable
//Retourne erreur 605 si indice est hors borne
ReelInfini obtenirLaValeurDUnElement(Matrice m,int indiceLigne,int indiceColonnne);

//==========
//Retourne NULL et erreur 603 si m est introuvable
//Retourne erreur 605 si indice est hors borne
Matrice obtenirUneColonne(Matrice m, int indiceColonne);

//==========
//Retourne NULL et erreur 603 si m est introuvable
//Retourne erreur 605 si indice est hors borne
Matrice obtenirUneLigne(Matrice m, int indiceLigne);

//==========
//Retourne NULL et erreur 603 si m est introuvable
//Retourne NULL et erreur 605 si indice est hors borne
Matrice obtenirUneSousMatrice(Matrice m, int indicePremiereLigne,
     int indicePremiereColonne,int indiceDerniereLigne,int indiceDerniereColonne);

//==========
//Retourne 1 si m est une matrice carree
//Retourne 0 si non
//Retourn -1 et erreur 603 si m est non trouvable
int estUneMatriceCarree(Matrice m);

//==========
//Retourne 1 si m est une matrice triangulaire suprieure
//Retourne 0 si non
//Retourn -1 et erreur 603 si m est non trouvable
//Retourn -1 et erreur 607 s'il y a une erreur du ReelInfini
int estUneMatriceTriangulaireSuperieure(Matrice m);

//==========
//Retourne 1 si m est une matrice triangulaire infrieure
//Retourne 0 si non
//Retourn -1 et erreur 603 si m est non trouvable
//Retourn -1 et erreur 607 s'il y a une erreur du ReelInfini
int estUneMatriceTriangulaireInferieure(Matrice m);

//==========
//Retourne 1 si m est une matrice diagonale
//Retourne 0 si non
//Retourn -1 et erreur 603 si m est non trouvable
int estUneMatriceDiagonale(Matrice m);

//==========
//Retourne 1 si m est une matrice symtrique
//Retourne 0 si non
//Retourn -1 et erreur 603 si m est non trouvable
int estUneMatriceSymetrique(Matrice m);

//==========
//Retourne 1 si m est une matrice ligne
//Retourne 0 si non
//Retourne -1 et erreur 603 si m est non trouvable
int estUneMatriceLigne(Matrice m);

//==========
//Retourne 1 si m est une matrice colonne
//Retourne 0 si non
//Retourne -1 et erreur 603 si m est non trouvable
int estUneMatriceColonne(Matrice m);

/*===============================Manipulation de la matrice==========================*/

//==========
//Retourne NULL et erreur 603 si m est introuvable
//Retourne m et erreur 605 si indice est hors borne
Matrice fixerLaValeurDUnElement(Matrice m,int indiceLigne,int indiceColonne,ReelInfini valeurElement);

//==========
//Retourne NULL et erreur 603 si m est introuvable
//Retourne m et erreur 603 si colonne est introuvable
//Retourne m et erreur 605 si indice est hors borne
//Retourne m et erreur 604 si les tailles ne sont pas compatibles
//Retourne m et erreur 610 si colonne n'est pas un vecteur colonne
Matrice fixerUneColonne(Matrice m,Matrice colonne,int indiceColonne);

//==========
//Retourne NULL et erreur 603 si m est introuvable
//Retourne m et erreur 603 si ligne est introuvable
//Retourne m et erreur 605 si indice est hors borne
//Retourne m et erreur 604 si les tailles ne sont pas compatibles
//Retourne m et erreur 606 si ligne n'est pas un vecteur ligne
Matrice fixerUneLigne(Matrice m,Matrice ligne,int indiceLigne);

//==========
//Retourne NULL et erreur 603 si m est introuvable
//Retourne m et erreur 605 si indice est hors borne
Matrice fixerUneSousMatrice(Matrice m,Matrice mAInserer,int indicePremiereLigne,
      int indicePremiereColonne, int indiceDerniereLigne,int indiceDerniereColonne);

//==========
//Retourne NULL et erreur 603 si m est introuvable
//Retourne m et erreur 605 si indice est hors borne
Matrice permuterDeuxColonnes(Matrice m,int indiceColonne1,int indiceColonne2);

//==========
//Retourne NULL et erreur 603 si m est introuvable
//Retourne m et erreur 605 si indice est hors borne
Matrice permuterDeuxLignes(Matrice m,int indiceLigne1,int indiceLigne2);


/*===============================Calcul sur la Matrice===============================*/

//==========
//Retourne NULL et erreur 604 si es tailles de m1 et m2 ne sont pas compatibles
//Retourne NULL et erreur 603 si l'une des deux matrices(ou toutes les deux
//est(sont) introuvable(s)
Matrice additionnerDeuxMatrices(Matrice m1,Matrice m2);

//==========
//Retourne NULL et erreur 604 si es tailles de m1 et m2 ne sont pas compatibles
//Retourne NULL et erreur 603 si l'une des deux matrices(ou toutes les deux) est(sont) introuvable(s)
Matrice multiplierDeuxMatrices(Matrice m1,Matrice m2);

//==========
//Retourne NULL et erreur 604 si es tailles de m1 et m2 ne sont pas compativbles
//Retourne NULL et erreur 603 si l'une des deux matrices(ou toutes les deux) est(sont) introuvable(s)
Matrice multiplierTermeATermeDeuxMatrices(Matrice m1,Matrice m2);

//==========
//Retourne NULL et erreur 603 si m est introuvable
Matrice multiplierUneMatriceParUnScalaire(Matrice m,ReelInfini scalaire);

//==========
//Retourne NULL et erreur 603 si m est introuvable
//Retourne NULL et erreur 608 si m n'est pas factorisable
Matrice calculerLaMatriceDiagonale(Matrice m);

//==========
//Retourne NULL et erreur 603 si m est introuvable
//Retourne NULL et erreur 608 si m n'est pas factorisable
Matrice calculerLaMatriceTriangulaireSuperieure(Matrice m);

//==========
//Retourne NULL et erreur 603 si m est introuvable
//Retourne NULL et erreur 608 si m n'est pas factorisable
Matrice calculerLaMatriceTriangulaireInferieure(Matrice m);

//==========
//Retourne NULL et erreur 603 si m est introuvable
Matrice calculerLaMatriceTransposee(Matrice m);

//==========
//Retourne NULL et erreur 603 si m est introuvable
//Retourne -1 et erreur 609 si m n'est pas carre
//Retourne NULL si m n'est pas inversible
Matrice calculerLaMatriceInverse(Matrice m);

//==========
//Retourne 0 et erreur 608 si m n'est pas factorisable
//Retourne -1 et erreur 603 si m est introuvable
//Retourne -1 et erreur 609 si m n'est pas carre
//Retourne 1 si 0K
//int factoriserLU(Matrice A, Matrice* L, Matrice* U);

//==========
//Resoudre le systeme A*x=secondMembre
//Retourne -1 et erreur 603 si A ou secondMembre est introuvable
//Retourne -1 et erreur 606 si secondMembre n'est pas un vecteur
//Retourne -1 et erreur 604 si les tailles de A et secondMembre ne sont pas compatibles
//Retourne 0 s'il y a pas une solution
//Retourne 1 si OK
//int resoudreLeSysteme(Matrice A, Matrice secondMembre, Matrice* solution);

#endif


